import React, { useEffect, useState } from 'react';
import { Box, Text } from 'ink';
import Spinner from 'ink-spinner';
import SelectInput from 'ink-select-input';

import Table from '@common/components/SelectTable';
import { MigrationParams } from 'cli/commands/migration/migrate/index';

interface Props {
  migrate: (params: MigrationParams) => void;
}

enum MigrationStep {
  INIT,
  TABLE,
  ACTION,
  TABLE_CHANGES
}

const Root: React.FC<Props> = ({ migrate }) => {
  const [table, setTable] = useState('');
  const [action, setAction] = useState('');
  const [index, setIndex] = useState(0);
  const [component, setComponent] = useState('');

  const [tableChanges, setTableChanges] = useState('');
  const [currentStep, setCurrentStep] = useState(MigrationStep.INIT);

  const handleTable = (a: any) => {
    setTable(a);
    setCurrentStep(MigrationStep.ACTION);
  };

  const handleAction = (a: any) => {
    setAction(a);
    setComponent((text) => `${text}\n Table 1 is: ${tableA[index].renamed}`);
    setCurrentStep(MigrationStep.TABLE_CHANGES);
  };

  useEffect(() => {
    setTimeout(() => setCurrentStep(MigrationStep.TABLE), 2000);
  }, []);

  const handleSelect = (index: number) => {
    setComponent((text) => `${text}\n` + `We're going to have table view with two columns: ${tableA[index].newName}`);
    setCurrentStep(MigrationStep.ACTION);
  };

  const items = [
    {
      label: 'First',
      value: 'first',
    },
    {
      label: 'Second',
      value: 'second',
    },
    {
      label: 'Third',
      value: 'third',
    },
  ];

  const actions = [
    {
      label: 'Renamed',
      value: 'Renamed',
    },
    {
      label: 'Deleted',
      value: 'Deleted',
    },
  ];

  const tableA = [
    {
      missing: 'Table 1', newName: 'Table 2', deleted: false, renamed: 'Table 2',
    },
    {
      missing: 'Table 3', newName: 'Table 4', deleted: true, renamed: '',
    },
    {
      missing: 'Table 5', newName: 'Table 6', deleted: false, renamed: 'Table 4',
    },
  ];

  const testRender = () => {
    if (currentStep === MigrationStep.INIT) {
      return (
        <Text>
          <Spinner />
          ...progress
        </Text>
      );
    }

    if (currentStep === MigrationStep.TABLE) {
      return (
        <>
          <Text>We`re going to have table view with two columns</Text>
          <Table
            text="Conflicts"
            onDone={handleSelect}
            selectFrom={tableA}
          />
        </>
      );
    }

    if (currentStep === MigrationStep.ACTION) {
      return (
        <>
          <Text>Table 1 is:</Text>
          <SelectInput items={actions} onSelect={handleAction} />
        </>
      );
    }

    return (<></>);
  };

  return (
    <>
      <Box marginBottom={1}>
        <Text>{component}</Text>
      </Box>
      {testRender()}
    </>
  );
};

export default Root;
