import React, { useEffect, useState } from 'react';
import { Box, Text, useInput } from 'ink';

interface Props {
  text: React.ReactFragment;
  selectFrom: {missing: string, newName: string, deleted: boolean, renamed: string}[];
  onDone: (value: number) => void;
}

const SelectTable: React.FC<Props> = ({
  text,
  selectFrom,
  onDone,
}: React.PropsWithChildren<Props>) => {
  const [selectedIndex, setSelectedIndex] = useState(0);
  const [isDone, setDone] = useState(false);

  useInput((input, key) => {
    if (key.upArrow) {
      const updatedIndex = selectedIndex === 0 ? selectFrom.length - 1 : selectedIndex - 1;
      setSelectedIndex(updatedIndex);
    }
    if (key.downArrow) {
      const updatedIndex = selectedIndex === (selectFrom.length - 1) ? 0 : selectedIndex + 1;
      setSelectedIndex(updatedIndex);
    }

    if (key.return) {
      setDone(true);
    }
  });

  useEffect(() => {
    if (isDone) {
      onDone(selectedIndex);
    }
  }, [isDone]);

  const prefix = (
    <Box flexDirection="row">
      <Box marginRight={1}>
        {isDone ? (
          <Text>
            [
            <Text color="green">✓</Text>
            ]
          </Text>
        ) : (
          <Text>
            [
            <Text color="gray">?</Text>
            ]
          </Text>
        )}
      </Box>
      <Text>
        {text}
        {' '}
      </Text>
    </Box>
  );

  return (
    <>
      <Box flexDirection={isDone ? 'row' : 'column'}>
        {prefix}
        {isDone ? (
          <Text>{selectedIndex}</Text>
        ) : (
          <>
            <Box borderStyle="single" padding={-1} width="50%">
              <Box width="25%">
                <Text>Missing</Text>
              </Box>
              <Box width="25%">
                <Text>| New</Text>
              </Box>
              <Box width="25%">
                <Text>| Deleted</Text>
              </Box>
              <Box width="25%">
                <Text> | Renamed</Text>
              </Box>
            </Box>
            {selectFrom.map(({
              missing, newName, deleted, renamed,
            }, index) => (
              <Box key={index} padding={0} width="50%">
                <Box paddingLeft={1} width="25%">
                  <Text
                    color={index === selectedIndex ? 'green' : 'white'}
                  >
                    {index === selectedIndex ? '>' : '|'}
                    {' '}
                    {missing}
                  </Text>
                </Box>
                <Box width="25%">
                  <Text
                    color={index === selectedIndex ? 'green' : 'white'}
                  >
                    |
                    {' '}
                    {newName}
                  </Text>
                </Box>
                <Box width="25%">
                  <Text
                    color={index === selectedIndex ? 'green' : 'white'}
                  >
                    |
                    {' '}
                    {deleted && '✓'}
                  </Text>
                </Box>
                <Box width="25%">
                  <Text
                    color={index === selectedIndex ? 'green' : 'white'}
                  >
                    |
                    {' '}
                    {renamed || ''}
                  </Text>
                </Box>
              </Box>
            ))}
          </>
        )}
      </Box>
    </>
  );
};

export default SelectTable;
